@@ -11,7 +11,7 @@ class AIHandler:
             self.client = OpenAI(api_key=open_ai_api_key)
         except ImportError:
             raise ImportError(
-                "OpenAI package is not installed. Install it with 'pip install yougotmail[openai]'"
+                "OpenAI package is not installed. Install it with pip install 'yougotmail[openai]'"
             )
 
     def _content(self, content):
