@@ -36,6 +36,7 @@ class YouGotMail:
             client_id,
             client_secret,
             tenant_id,
+            open_ai_api_key=open_ai_api_key,
             mongo_url=mongo_url,
             mongo_db_name=mongo_db_name,
             email_collection=email_collection,
@@ -342,14 +343,14 @@ class YouGotMail:
         storage=None,
         schema={}
     ):
-        ai = self._get_ai()
+        # ai = self._get_ai()
         # Add custom validation logic specific to this function
         if range and (start_date or end_date or start_time or end_time):
             raise ValidationError(
                 "Cannot specify both 'range' and custom date/time parameters"
             )
 
-        return ai.ai_get_emails_with_structured_output(
+        return self.retrieve_emails.get_emails_with_structured_outputs(
             inbox=inbox,
             range=range,
             start_date=start_date,
